##      B&P Glossy Web 2.0 style Button 02
##      Version: 1.2  - [UPDATED: 2012-04-26]

#This version is identical to the version 1.0 in all functionality,
#I just replaced all deprecated functions to work in GIMP 2.7 - 2.8
#
# IT IS NOT COMPATIBLE WITH GIMP 2.6
#
#it creates a glossy Web2.0 style button
#Accepted parameters are dimensions, color,
#corner radius, Add optional text - size, font, color etc.
#
# Works best with dark colors and high saturation.
#
# Note:
# Developed and tested with Python 2.6.
#
# Original plugin released: 2010-Dec-07
#
# Andrei Roslovtsev
# www.bytes-and-pixels.com
# andrei[AT]bytes-and-pixels.com

# -----------------------------------------------------------
from gimpfu import *
import sys

def make_button(width_var,height_var,rounding,
                border,main_col,text_flag,
                text_size,button_txt,font,text_col,
                flatten_flag,states_flag,crop_flag):

    if font==None:
        font='Sans'

    margin = 10
    #border = 2
    
    # set image dimensions
    img_w = width_var+margin*2
    img_h = height_var+margin*2
        
    image = pdb.gimp_image_new(img_w, img_h, 0)
    pdb.gimp_image_set_filename(image, "Up_image")

    # reset colors to B&W
    pdb.gimp_context_set_default_colors()

    layer_group = pdb.gimp_layer_group_new(image)
    pdb.gimp_item_set_name(layer_group, 'Button_Layers')
    pdb.gimp_image_insert_layer(image, layer_group, None, 0)
    
    layer1 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'BG', 100, 0)
    layer2 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Border', 100, 0)
    pdb.gimp_image_insert_layer(image, layer1, layer_group, 0)
    pdb.gimp_image_insert_layer(image, layer2, layer_group, -1)
    
    display = pdb.gimp_display_new(image)
    gimp.pdb.gimp_edit_fill(layer1, 3)


# Create border layer -------------------------------------

    pdb.gimp_image_select_round_rectangle(image, 2, margin, margin, width_var,
                                          height_var, rounding, rounding)
    
    pdb.gimp_edit_bucket_fill(layer2, 0, 0, 100, 0, 0, 0, 0)

# Create gradient layer -------------------------------------

    # Check if needed gradient exists, then replace or create one
    if find_gradient_by_name("b&p_btn2_grad") == True:
        pdb.gimp_gradient_delete("b&p_btn2_grad")

    grad = pdb.gimp_gradient_new("b&p_btn2_grad")
    pdb.gimp_context_set_gradient(grad)

    # modify the gradient
    pdb.gimp_gradient_segment_set_left_color(grad, 0, (37,37,37), 100)
    pdb.gimp_gradient_segment_set_right_color(grad, 0, (140,140,140), 100)

    pdb.gimp_gradient_segment_range_split_midpoint(grad, 0, 1)

    pdb.gimp_gradient_segment_set_middle_pos(grad, 0, 0.45)
    pdb.gimp_gradient_segment_set_middle_pos(grad, 1, 0.55)

    #Save original selection (outline of the button's border)
    outer_shape = pdb.gimp_selection_save(image)

    #fix for when rounding is < 2
    if rounding < border:
        rounding=border
        
    pdb.gimp_image_select_round_rectangle(image, 2, margin+border, margin+border,
                                          width_var-border*2, height_var-border*2,
                                          rounding-border, rounding-border)
    
    #Save inner selection (inside of the button)
    inner_shape = pdb.gimp_selection_save(image)

    pdb.gimp_edit_blend(layer2, 3, 0, 0, 100, 0, 0, 0,
                        1, 4, 2, 1, img_w/2, 0, img_w/2, img_h)

#Add color layer --------------------------------------------------------
    layer3 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Color', 100, 5)
    pdb.gimp_image_insert_layer(image, layer3,layer_group, 0)
    #pdb.gimp_selection_load(orig_shape)

    pdb.gimp_context_set_foreground(main_col)
    pdb.gimp_edit_bucket_fill(layer3, 0, 0, 100, 0, 0, 0, 0)
    
#Add specular highlight layer (top thin line) --------------------------------------------------------
    layer4 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Spec Top', 100, 0)
    pdb.gimp_image_insert_layer(image, layer4, layer_group, 1) 
    
    if img_w-(margin+border+rounding)*2>0:

        pdb.gimp_image_select_rectangle(image, 2, margin+border+rounding,
                                        margin+border, img_w-(margin+border+rounding)*2,
                                        3)
        pdb.gimp_context_swap_colors()
        pdb.gimp_context_set_gradient("FG to Transparent")

        #Apply our gradient to selection
        pdb.gimp_edit_blend(layer4, 2, 0, 1,
                            100, 0, 0, 0, 0,
                            0, 0, 1, img_w/2, 0, img_w-(margin+border+rounding), 0)
        
        pdb.plug_in_blur(image, layer4)

#Add lower highlight layer 1 (hazy big gradient) --------------------------------------------------------
    layer5 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Grad 01', 100, 0)
    pdb.gimp_image_insert_layer(image, layer5, layer_group, 2)

    #load outer rect selection
    pdb.gimp_image_select_item(image, 2, inner_shape)

    #run a series of gradients
    x = img_w/2  #this is constant - center of the button
    y1 = margin + height_var*3 #gradient start  <-- img_h*3
    y2 = img_h/2 #gradient end
    y1_debug = ""
    y2_debug = ""
    
    #apply gradient 6 times:
    i=0
    while i<6:
        y1_debug += "," + str(y1)
        y2_debug += "," + str(y2)
        pdb.gimp_edit_blend(layer5, 2, 0, 2,
                            100, 0, 0, 0, 1,
                            3, 1, 1, x, y1, x, y2)
        i = i+1
        y1 = (y1-img_h/2)/1.2 + img_h/2+margin
        y2 = y2 + (y2-margin)/10
    
#Add lower highlight layer 2 (sharper narrow gradient) --------------------------------------------------------
    layer6 = pdb.gimp_layer_copy(layer5,0)
    pdb.gimp_image_insert_layer(image, layer6, layer_group, 2)

    pdb.gimp_selection_none(image)
    
##    tmp = pdb.gimp_drawable_transform_scale(layer6, 0, img_h*0.70, img_w, img_h, 0, 2, 0, 3, 0)
    pdb.gimp_item_transform_scale(layer6, 0, img_h*0.70, img_w, img_h)

    pdb.gimp_layer_translate(layer6, 0, -1*(margin))

##    pdb.gimp_selection_load(inner_shape)
    pdb.gimp_image_select_item(image, 2, inner_shape)
    pdb.gimp_selection_invert(image)
    pdb.gimp_edit_clear(layer6)
    pdb.gimp_selection_none(image)

    #hide original gradient (will be used for hover effect)
##    pdb.gimp_drawable_set_visible(layer5, 0)
    pdb.gimp_item_set_visible(layer5, 0)

# Text Layer (optional) --------------------------------------------
    if text_flag == 1:
        layer7 = pdb.gimp_text_layer_new(image, button_txt, font, text_size, 0)
        pdb.gimp_image_insert_layer(image, layer7, layer_group, 0)
        pdb.gimp_text_layer_set_color(layer7, text_col)
        
        txt_x = (image.width-layer7.width)/2
        txt_y = (image.height-layer7.height)/2
        layer7.set_offsets(txt_x, txt_y)

# Create state images -----------------------------------------------
    if states_flag == 1:
        over_image = pdb.gimp_image_duplicate(image)
        down_image = pdb.gimp_image_duplicate(image)

        pdb.gimp_image_set_filename(over_image, "Over_image")
        pdb.gimp_image_set_filename(down_image, "Down_image")

        display = pdb.gimp_display_new(over_image)
        display = pdb.gimp_display_new(down_image)

        # Edit "Over" state
        o_layer5 = find_layer_by_name(over_image,"Grad 01")
        o_layer1 = find_layer_by_name(over_image,"Border")
        
        pdb.gimp_item_set_visible(o_layer5, 1)


        # Edit "Down" state
        d_layer4 = find_layer_by_name(down_image,"Color")
        d_layer1 = find_layer_by_name(down_image,"Border")
        pdb.gimp_image_set_active_layer(down_image, d_layer4)
        pdb.gimp_hue_saturation( d_layer4, 0, -50, 0, -50)

# Flatten Image routine ---------------------------------------------
    if flatten_flag == 1:
        pdb.gimp_image_remove_layer(image, layer1)
        pdb.gimp_image_remove_layer(image, layer5)
        layer1 = pdb.gimp_image_merge_visible_layers(image, 0)

        if states_flag == 1:
            pdb.gimp_image_remove_layer(over_image, find_layer_by_name(over_image,"BG"))
            o_layer1 = pdb.gimp_image_merge_visible_layers(over_image, 0)

            pdb.gimp_image_remove_layer(down_image, find_layer_by_name(down_image,"BG"))
            pdb.gimp_image_remove_layer(down_image, find_layer_by_name(down_image,"Grad 01"))
            d_layer1 = pdb.gimp_image_merge_visible_layers(down_image, 0)
        
    pdb.gimp_image_set_active_layer(image, layer1)

# Autocrop Image routine ---------------------------------------------
    if crop_flag == 1:
        if flatten_flag == 0:
            pdb.plug_in_autocrop(image, layer2)
        else:
            pdb.plug_in_autocrop(image, layer1)
        if states_flag == 1:
            pdb.plug_in_autocrop(over_image, o_layer1)
            pdb.plug_in_autocrop(down_image, d_layer1)

    # reset colors to B&W
    pdb.gimp_context_set_default_colors()
#--------------------------------------------------------------------
#Helper functions
#Author:  Joao S. O. Bueno
#source:  http://www.gimpusers.com/forums/gimp-user/
#    11870-set-active-layer-by-referring-to-its-name

# Edited by Andrei Roslovtsev:
# 2012-04-25
# to address limitation of 2.7+ regarding layer groups:
# see this post:
# http://stackoverflow.com/questions/10221926/how-get-sublayers-from-group-layer-with-python-in-gimp
def find_layer_by_name (image, name):
    for layer in image.layers:
        if pdb.gimp_item_is_group(layer):
            gr = layer
            gr_items = pdb.gimp_item_get_children(layer)
            for index in gr_items[1]:
                item = gimp.Item.from_id(index)
                if item.name == name:
                    return item


        if layer.name == name:
            return layer
            return None

def find_gradient_by_name (name):
    num_gradients, gradient_list = pdb.gimp_gradients_get_list("")

    if name in gradient_list:
        return True
    return False
   

#---------------------------------------------------------------------
register(
    "Glossy_Button_02",
    N_("Create a glossy button"),
    "Create a glossy Web2.0 style button",
    "Anadrei Roslovtsev",
    "www.bytes-and-pixels.com",
    "2010",
    N_("_Glossy button 02"),
    #"RGB*, GRAY*",
    "",
    [
        (PF_SPINNER, "width_var", "Button Width:", 140, (1, 2000, 1)),
        (PF_SPINNER, "height_var", "Button Height:", 50, (1, 2000, 1)),
        (PF_SPINNER, "rounding", "Corner Radius:", 12, (0, 100, 1)),
        (PF_SPINNER, "border", "Border Width:", 2, (0, 100, 1)),
        (PF_COLOR, "main_col", "Color:", (84, 66, 217) ),
        (PF_TOGGLE, "text_flag", "Add Text:", 1),
        (PF_SPINNER, "text_size", "Text Size (px):", 20, (1, 2000, 1)),
        (PF_STRING, "button_txt", "Text:", "Download"),
        (PF_FONT, "font", "FONT:", 0),
        (PF_COLOR, "text_col", "Text Color:", (255, 255, 255) ),
        (PF_TOGGLE, "flatten_flag", "Flatten Image", 0),
        (PF_TOGGLE, "states_flag", "Generate Button States", 0),
        (PF_TOGGLE, "crop_flag", "Autocrop Result", 0)
    ],
    [],
    make_button,
    #menu="<Image>",
    menu="<Toolbox>/Xtns/Web Page Themes/www.bytes-and-pixels.com",
    domain=("gimp20-python", gimp.locale_directory)
    )

main()
